# -*- coding: utf-8 -*-

"""The application's Globals object"""

__all__ = ['Globals']
from libacr.views.manager import ViewsManager
from libacr.plugins.manager import PluginsManager
from libacr.mediaWorker import start_media_worker


class Globals(object):
    """Container for objects available throughout the life of the application.

    One instance of Globals is created during application initialization and
    is available during requests via the 'app_globals' variable.

    """

    def __init__(self):
        self.acr_viewmanager = ViewsManager()
        self.plugins = PluginsManager()
        """Do nothing, by default."""
        from turbomail.adapters import tm_pylons
        tm_pylons.start_extension()



from tg import expose, flash, require, request, redirect, tmpl_context, TGController, config, validate
from tg.controllers import WSGIAppController
import libacr
from libacr.lib import url as acr_url

from libacr.controllers.admin.base import BaseAdminController
from libacr.plugins.base import AdminEntry, AcrPlugin, plugin_expose
from repoze.what import predicates
from tw.api import WidgetsList
import tw.forms as twf

class OldPasswordValidator(twf.validators.FancyValidator):
    def validate_python(self, value, state):
        if not request.identity["user"].validate_password(value):
            raise twf.validators.Invalid("wrong pass", value, state)

class ChPassForm(twf.TableForm):
    class fields(WidgetsList):
        oldpass = twf.PasswordField(label_text="Current Password",validator = OldPasswordValidator(not_empty=True))
        newpass1 = twf.PasswordField(label_text="New Password", validator = twf.validators.String(not_empty=True))
        newpass2 = twf.PasswordField(label_text="Confirm New Password", validator = twf.validators.String(not_empty=True))

    class NewPassValidator(twf.validators.Schema):
        chained_validators = [twf.validators.FieldsMatch('newpass1','newpass2')]
    validator=NewPassValidator()

chpassform = ChPassForm(submit_text="Change")

class ChpasswdController(BaseAdminController):

    @plugin_expose('index')
    @require(predicates.not_anonymous())
    def index(self, **kw):
        return dict(form=chpassform,values=kw)

    @expose()
    @require(predicates.not_anonymous())
    @validate(form=chpassform,error_handler=index)
    def change(self,**kw):
        request.identity["user"].password = kw['newpass1']
        flash("password changed succesfully")
        return redirect(acr_url('/admin'))

class ChangePass(AcrPlugin):
    uri = 'chpasswd'

    def __init__(self):
        self.admin_entries = [AdminEntry(self, 'Change My Password', 'index', icon='Gnome-Dialog-Password-64.png', section="General Settings")]
        self.controller = ChpasswdController()



# -*- coding: utf-8 -*-
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

version='0.2'

setup(
    name='acr',
    version=version,
    description='ACRCms is an open source Python web content management system based on Turbogears and libacr',
    long_decription="""\
The ACR project has born in 2009 as a library to be imported inside any Turbogears application to provide a
way to quickly create web pages and manage their content.
After releasing it as open source the AXANT team thought that it would have been useful to provide an example
on how to use it and created ACRCms which is a Python web content management system based on libacr.

Currently ACRCms supports:

    * Multilanguage Content
    * Content versioning and reverting
    * Image and HTML5 Video Galleries
    * News/Blog Sections
    * User permissions management
    * Autogenerated Vertical/Horizontal Menus
    * Internal Search Engine
    * Youtube Videos embedding
    * Remote Disk to upload and manage files
    * RSS Integration (import and export content using RSS)
    * Twitter Integration (import your tweets)
    * Static and Dynamic Google Maps
    * Comments
    * Automatic Forms generation with email submission of data
    * An extensible plugin architecture to add new functions any time
    * Uservoice feedback plugin
    * Themes directly editable within the Administration area
    * Google Analytics plugin for stats
    * Various king of slideshows

Plans for the future include layout management interface and a rich set of plugins,
if you want to contribute feel free to send an email at: tech [at] axant.it
""",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: TurboGears",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='tg2 cms cmf web',
    author='AXANT',
    author_email='tech@axant.it',
    url='http://www.acrcms.org',
    license='GPL',
    install_requires=[
        "TurboGears2 >= 2.0b7",
        "Babel >=0.9.4",
        "zope.sqlalchemy >= 0.4 ",
        "repoze.tm2 >= 1.0a4",
        "repoze.what-quickstart >= 1.0",
        "feedparser",
        "tw.jquery",
        "tw.tinymce",
        "BeautifulSoup",
        "PIL",
        "sprox",
        "tgext.admin",
        "libacr",
    ],
    zip_safe=False,
    setup_requires=["PasteScript >= 1.7"],
    paster_plugins=['PasteScript', 'Pylons', 'TurboGears2', 'tg.devtools'],
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=['WebTest', 'BeautifulSoup'],
    package_data={'acr': ['i18n/*/LC_MESSAGES/*.mo',
                                 'templates/*/*',
                                 'public/*/*',
                                 'acr_plugins/*/*.html',
                                 'acr_plugins/*/static/*.*']},
    message_extractors={'acr': [
            ('**.py', 'python', None),
            ('templates/**.mako', 'mako', None),
            ('templates/**.html', 'genshi', None),
            ('public/**', 'ignore', None)]},

    entry_points="""
    [paste.app_factory]
    main = acr.config.middleware:make_app

    [paste.app_install]
    main = pylons.util:PylonsInstaller
    """,
)



# -*- coding: utf-8 -*-
"""WSGI middleware initialization for the acr application."""

from acr.config.app_cfg import base_config
from acr.config.environment import load_environment


__all__ = ['make_app']

# Use base_config to setup the necessary PasteDeploy application factory. 
# make_base_app will wrap the TG2 app with all the middleware it needs. 
make_base_app = base_config.setup_tg_wsgi_app(load_environment)


def make_app(global_conf, full_stack=True, **app_conf):
    """
    Set acr up with the settings found in the PasteDeploy configuration
    file used.
    
    :param global_conf: The global settings for acr (those
        defined under the ``[DEFAULT]`` section).
    :type global_conf: dict
    :param full_stack: Should the whole TG2 stack be set up?
    :type full_stack: str or bool
    :return: The acr application with all the relevant middleware
        loaded.
    
    This is the PasteDeploy factory for the acr application.
    
    ``app_conf`` contains all the application-specific settings (those defined
    under ``[app:main]``.
    
   
    """
    app = make_base_app(global_conf, full_stack=True, **app_conf)
    
    # Wrap your base TurboGears 2 application with custom middleware here
    
    return app



# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import expose, flash, require, request, redirect, tmpl_context
from tg import url as tgurl
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from pylons.controllers.util import abort

from repoze.what import predicates

from acr.lib.base import BaseController, get_page_from_urllist, url
from acr.model import DBSession, Page
from acr import model

from libacr.controllers.cms import AcrRootController
from libacr.controllers.rdisk import RDiskController
from acr.controllers.error import ErrorController

from turbomail import Message


__all__ = ['RootController']

class RootController(BaseController):
    acr = AcrRootController()
    rdisk = RDiskController()
    error = ErrorController()

    @expose()
    def index(self, *args, **kw):
        return redirect('/acr/index')

    @expose('acr.templates.login')
    def login(self, came_from=url('/')):
        """Start the user login."""
        login_counter = request.environ['repoze.who.logins']
        if login_counter > 0:
            flash(_('Wrong credentials'), 'warning')
        return dict(page=None, login_counter=str(login_counter),
                    came_from=came_from)

    @expose()
    def post_login(self, came_from=url('/')):
        """
        Redirect the user to the initially requested page on successful
        authentication or redirect her back to the login page if login failed.
        
        """
        if not request.identity:
            login_counter = request.environ['repoze.who.logins'] + 1
            redirect(tgurl('/login', came_from=came_from, __logins=login_counter))
        userid = request.identity['repoze.who.userid']
        flash(_('Welcome back, %s!') % userid)
        redirect(came_from)

    @expose()
    def post_logout(self, came_from=url('/')):
        """
        Redirect the user to the initially requested page on logout and say
        goodbye as well.
        
        """
        flash(_('We hope to see you soon!'))
        redirect(came_from)



# -*- coding: utf-8 -*-
"""The application's model objects"""

from zope.sqlalchemy import ZopeTransactionExtension
from sqlalchemy.orm import scoped_session, sessionmaker
#from sqlalchemy import MetaData
from sqlalchemy.ext.declarative import declarative_base
from libacr.model import init_acr_model
from acr.lib.multisite import AcrMultiSiteEngine

# Global session manager: DBSession() returns the Thread-local
# session object appropriate for the current web request.
maker = sessionmaker(autoflush=True, autocommit=False,
                     extension=ZopeTransactionExtension())
DBSession = scoped_session(maker)
multisite_engine = AcrMultiSiteEngine()

# Base class for all of our model classes: By default, the data model is
# defined with SQLAlchemy's declarative extension, but if you need more
# control, you can switch to the traditional method.
DeclarativeBase = declarative_base()

# There are two convenient ways for you to spare some typing.
# You can have a query property on all your model classes by doing this:
# DeclarativeBase.query = DBSession.query_property()
# Or you can use a session-aware mapper as it was used in TurboGears 1:
# DeclarativeBase = declarative_base(mapper=DBSession.mapper)

# Global metadata.
# The default metadata is the one from the declarative base.
metadata = DeclarativeBase.metadata

# If you have multiple databases with overlapping table names, you'll need a
# metadata for each database. Feel free to rename 'metadata2'.
#metadata2 = MetaData()

#####
# Generally you will not want to define your table's mappers, and data objects
# here in __init__ but will want to create modules them in the model directory
# and import them at the bottom of this file.
#
######
def init_model(engine):
    """Call me before using any of the tables or classes in the model."""
    DBSession.configure(bind=multisite_engine)
    # If you are using reflection to introspect your database and create
    # table objects for you, your tables must be defined and mapped inside
    # the init_model function, so that the engine is available if you
    # use the model outside tg2, you need to make sure this is called before
    # you use the model.

    #
    # See the following example:

    #global t_reflected

    #t_reflected = Table("Reflected", metadata,
    #    autoload=True, autoload_with=engine)

    #mapper(Reflected, t_reflected)

# Import your model modules here.
from auth import User, Group, Permission
Content, ContentData, Tag, Page, Slice, View, Comment = init_acr_model(DBSession, DeclarativeBase, User, Group)
